import { assign, createCanvas, WindCore, defaultOptions, isArray, formatData, warnLog } from 'wind-core';
export { Field } from 'wind-core';
import * as ol from 'openlayers';

const _options = {
  windOptions: {}
};
class OlWind extends ol.layer.Image {
  constructor(data, options = {}) {
    const opt = assign({}, _options, options);
    super(options);
    this.options = opt;
    this.wind = null;
    this.type = "IMAGE";
    this.start = this.start.bind(this);
    this.stop = this.stop.bind(this);
    this.pickWindOptions();
    const sourceOptions = {
      logo: options.logo,
      state: options.state,
      attributions: options.attributions,
      resolutions: options.resolutions,
      canvasFunction: this.canvasFunction.bind(this),
      ratio: options.hasOwnProperty("ratio") ? options.ratio : 1
    };
    const source = new ol.source.ImageCanvas(sourceOptions);
    if (!source.getAttributions2) {
      source.getAttributions2 = () => source.getAttributions();
    }
    this.setSource(source);
    if (data) {
      this.setData(data, options.fieldOptions);
    }
  }
  /**
   * append layer to map
   * @param map
   */
  appendTo(map) {
    if (map) {
      this.setMap(map);
    } else {
      throw new Error("not map object");
    }
  }
  start() {
    if (this.wind) {
      this.wind.start();
    }
  }
  stop() {
    if (this.wind) {
      this.wind.stop();
    }
  }
  canvasFunction(extent, resolution, pixelRatio, size, proj) {
    this.pixelRatio = pixelRatio;
    if (!this.canvas) {
      this.canvas = createCanvas(size[0], size[1], 1, null);
    } else {
      this.canvas.width = size[0];
      this.canvas.height = size[1];
    }
    if (this.canvas) {
      this.render(this.canvas);
    }
    return this.canvas;
  }
  getContext() {
    if (this.canvas === null)
      return;
    return this.canvas.getContext("2d");
  }
  /**
   * render windy layer
   * @param canvas
   * @returns {OlWind}
   */
  render(canvas) {
    const map = this.getMap();
    if (!this.getData() || !map)
      return this;
    const opt = this.getWindOptions();
    if (canvas && !this.wind) {
      const data = this.getData();
      const ctx = this.getContext();
      if (ctx) {
        this.wind = new WindCore(ctx, opt, data);
        this.wind.project = this.project.bind(this);
        this.wind.unproject = this.unproject.bind(this);
        this.wind.intersectsCoordinate = this.intersectsCoordinate.bind(this);
        this.wind.postrender = () => {
          this.changed();
        };
      }
    }
    if (this.wind) {
      this.wind.prerender();
      this.wind.render();
    }
    return this;
  }
  project(coordinate) {
    const map = this.getMap();
    const pixel = map.getPixelFromCoordinate(ol.proj.transform(coordinate, "EPSG:4326", this.viewProjection));
    return [pixel[0] * this.pixelRatio, pixel[1] * this.pixelRatio];
  }
  unproject(pixel) {
    const map = this.getMap();
    const coordinates = map.getCoordinateFromPixel(pixel);
    return ol.proj.transform(coordinates, this.viewProjection, "EPSG:4326");
  }
  /**
   * TODO: 空间判断出错，需要修复
   * @param coordinate
   */
  intersectsCoordinate(coordinate) {
    const map = this.getMap();
    if (!map)
      return false;
    const view = map.getView();
    const size = map.getSize();
    if (view && size) {
      const extent = view.calculateExtent([size[0] * this.pixelRatio, size[1] * this.pixelRatio]);
      return ol.extent.containsCoordinate(extent, ol.proj.transform(coordinate, "EPSG:4326", this.viewProjection));
    }
    return false;
  }
  pickWindOptions() {
    Object.keys(defaultOptions).forEach((key) => {
      if (key in this.options) {
        if (this.options.windOptions === void 0) {
          this.options.windOptions = {};
        }
        this.options.windOptions[key] = this.options[key];
      }
    });
  }
  /**
   * get wind layer data
   */
  getData() {
    return this.field;
  }
  /**
   * set layer data
   * @param data
   * @param options
   * @returns {OlWind}
   */
  setData(data, options = {}) {
    if (data && data.checkFields && data.checkFields()) {
      this.field = data;
    } else if (isArray(data)) {
      this.field = formatData(data, options);
    } else {
      console.error("Illegal data");
    }
    const map = this.getMap();
    if (map && this.canvas && this.field) {
      this?.wind?.updateData(this.field);
      this.render(this.canvas);
    }
    return this;
  }
  updateParams(options = {}) {
    warnLog("will move to setWindOptions");
    this.setWindOptions(options);
    return this;
  }
  getParams() {
    warnLog("will move to getWindOptions");
    return this.getWindOptions();
  }
  setWindOptions(options) {
    const beforeOptions = this.options.windOptions || {};
    this.options = assign(this.options, {
      windOptions: assign(beforeOptions, options || {})
    });
    if (this.wind) {
      this.wind.setOptions(this.options.windOptions);
      this.wind.prerender();
    }
  }
  getWindOptions() {
    return this.options.windOptions || {};
  }
  getProjection() {
    let projection;
    const map = this.getMap();
    if (map) {
      projection = map.getView() && map.getView().getProjection();
    } else {
      projection = "EPSG:3857";
    }
    return projection;
  }
  /**
   * set map
   * @param map
   */
  setMap(map) {
    if (!map && this.wind) {
      this.wind.stop();
    }
    this.set("originMap", map);
    this.viewProjection = this.getProjection();
    return super.setMap(map);
  }
  /**
   * get map
   */
  getMap() {
    return this.get("originMap");
  }
  getType() {
    return this.type;
  }
}
const WindLayer = OlWind;

export { WindLayer, OlWind as default };
//# sourceMappingURL=ol-wind.esm.js.map
